@@ -6,8 +6,6 @@
 #     Copyright (C) 2001-2007 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
-#   $Id: Primitive.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
-#
 # G-language GAE is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public
 # License as published by the Free Software Foundation; either
@@ -31,10 +29,10 @@ package G::DB::BDB;
 use strict;
 use DBI;
 use SubOpt;
-use G::Skyline;
 use G::DB::SDB;
 
-require G::DB::BDBI;
+require G::IO;
+require G::DB::Handler;
 require Exporter;
 
 our @ISA = qw(Exporter);
@@ -92,7 +90,13 @@ sub db_load{
     }
 
     if($opt eq 'dbi'){
-	bless $ref, 'G::DB::BDBI';
+	bless $ref, 'G::DB::Handler';
+    }elsif($opt eq 'vm'){
+	if(sdb_exists("sdb_$name")){
+	    return sdb_load("sdb_$name");
+	}else{
+	    die("SDB: Data \"$name\" not found. Aborting.");
+	}
     }else{
 	my $table = $name;
 	my $flag = 0;
@@ -100,7 +104,16 @@ sub db_load{
 	unless($database){
 	    $table = db_exists($name);
 	    $flag = 1;
-	    die("BDB: Table \"$name\" not found. Aborting.") unless($table);
+
+	    unless($table){
+		if(sdb_exists("sdb_$name")){
+		    return sdb_load("sdb_$name");
+		}elsif(sdb_exists($name)){
+		    return sdb_load($name);
+		}else{
+		    die("BDB: Table \"$name\" not found. Aborting.");
+		}
+	    }
 	}
 
 	if($opt eq 'clone'){
@@ -110,9 +123,15 @@ sub db_load{
 	}
 	
 	unless($ref =~ /G::/){
-	    bless $ref, 'G::DB::BDBI';
+	    bless $ref, 'G::DB::Handler';
 	}
 	
+	if(length $primarykey < 1){
+	    my @primary_keys  = $dbh->primary_key(undef, undef, $table);
+	    $primarykey = shift @primary_keys;
+	}
+
+	$ref->set_primarykey($primarykey);
 	$ref->set_table($table);
     }
 
@@ -170,11 +189,18 @@ sub db_path{
 sub db_save{
     my $ref = shift;
     my $tablename = shift || "scratch";
+    my $vm = shift;
     my $dbh = db_dbi();
 
     my $name = $tablename;
     $tablename = 'gb_' . $tablename if (ref($ref) =~ /^G::/);
     my $depth = nested_hash_depth($ref);
+
+    if($depth < 2 || $vm eq 'vm'){
+	sdb_save($ref, "sdb_$name");
+	return;
+    }
+
     my @keys = nested_hash_keys($ref, $depth);
 
     my $createtableline = '';
@@ -297,7 +323,7 @@ sub _load{
     my $result = {};
     my %lonely;
 
-    $result = new G::Skyline("blessed") if($table =~ /^gb_/);
+    $result = new G::IO("blessed") if($table =~ /^gb_/);
 
     if(length $primarykey < 1){
 	my @primary_keys  = $dbh->primary_key(undef, undef, $table);
@@ -350,7 +376,7 @@ sub _clone{
     my $primarykey = shift;
 
     my $result = {};
-    $result = new G::Skyline("blessed") if($table =~ /^gb_/);
+    $result = new G::IO("blessed") if($table =~ /^gb_/);
 
     if(length $primarykey < 1){
 	my @primary_keys  = $dbh->primary_key(undef, undef, $table);
