@@ -38,7 +38,7 @@ use G::Seq::Primitive;
 
 require Exporter;
 
-our @ISA = qw(Exporter AutoLoader);
+our @ISA = qw(Exporter);
 our @EXPORT = qw(
 	     run_glimmer
 	     annotate_with_glimmer
