@@ -34,10 +34,10 @@ package G::IO::EmblI;
 use strict;
 use G::Messenger;
 
-require G::Prelude;
+require G::IO::Handler;
 require G::IO::GenBankO;
 
-our @ISA = qw(G::Prelude G::IO::GenBankO);
+our @ISA = qw(G::IO::Handler G::IO::GenBankO);
 
 our $VERSION = '1.0';
 
@@ -375,8 +375,7 @@ G::IO::EmblI
    
 =head1 DESCRIPTION
 
- This module is what is formerly known as 'Prelude'. 
- GenBank input parser.
+ EMBL input parser.
  Intended for internal use only.
 
 =back
