@@ -34,12 +34,6 @@ package G::IO::GenBankI;
 use strict;
 use G::Messenger;
 
-use vars qw($VERSION @ISA @INC $AUTOLOAD);
-
-@ISA = qw();
-
-$VERSION = '1.0';
-
 my $infile;
 
 #::::::::::::::::::::::::::::::
@@ -370,7 +364,6 @@ G::IO::GenBankI
    
 =head1 DESCRIPTION
 
- This module is what is formerly known as 'Prelude'. 
  GenBank input parser.
  Intended for internal use only.
 
