@@ -107,8 +107,8 @@ sub msg_gimv{
 	    $data{$time}->display('\G-language\title.jpg');
 	    $data{$time}->display_wait($filename);
 	}else{
-	    $gimv = 'mozilla' if ($file =~ /html$/ && $gimv eq 'gimv');
-	    $gimv = 'mozilla' if ($file =~ /svg$/ && $gimv eq 'gimv');
+	    $gimv = 'firefox' if ($file =~ /html$/ && $gimv eq 'gimv');
+	    $gimv = 'firefox' if ($file =~ /svg$/ && $gimv eq 'gimv');
 	    $file = 'file://' . getcwd() . '/' . $file if ($gimv eq 'mozilla');
 	    system("$gimv $file &");
 	}
