@@ -120,6 +120,7 @@ __DATA__
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
  History:
+   20071023-01 speed up by combining rough/detailed searches
    20070707-01 added -filter option
    20060711-01 added purine and keto options
    20060707-01 calculation is now based on single bp resolution rather than with windows
@@ -135,7 +136,7 @@ sub find_ori_ter {
     &opt_default(output=>"stdout", purine=>0, keto=>0, window=>4096);
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
-    my $seq = $gb->{SEQ};
+    my $len = length($gb->{SEQ});
     my $output = opt_val("output");
     my $purine = opt_val("purine");
     my $keto = opt_val("keto");
@@ -146,7 +147,7 @@ sub find_ori_ter {
 
     my ($max, $min);
     if(length $lowpass){
-	my $window = int(length($gb->{SEQ}) / $power);
+	my $window = int($len / $power);
 	my @gcskew = gcskew($gb, -output=>"/dev/null", -window=>$window, -purine=>$purine, -keto=>$keto);
 
 	while(scalar @gcskew > $power){
@@ -181,27 +182,47 @@ sub find_ori_ter {
 	$max = ($maxi + 1) * $window - int($window/2);
 	$min = ($mini + 1) * $window - int($window/2);
     }else{
+	local *peak_search = sub{
+	    my $seq = shift;
 
-	if($purine){
-	    $seq =~ tr/atgcn/02021/;
-	}elsif($keto){
-	    $seq =~ tr/atgcn/20021/;
-	}else{
-	    $seq =~ tr/atgcn/11021/;
-	}
-
-	my (@data, $val, $i);
-	for($i = 0; $i <= length($seq); $i ++){
-	    if(substr($seq, $i, 1) =~ /^\d$/){
-		$val += substr($seq, $i, 1) - 1;
+	    if($purine){
+		$seq =~ tr/atgcn/02021/;
+	    }elsif($keto){
+		$seq =~ tr/atgcn/20021/;
+	    }else{
+		$seq =~ tr/atgcn/11021/;
+	    }
+	    
+	    my (@data, $val, $i);
+	    for($i = 0; $i <= length($seq); $i ++){
+		if(substr($seq, $i, 1) =~ /^\d$/){
+		    $val += substr($seq, $i, 1) - 1;
+		}
+		push(@data, $val);
 	    }
-	    push(@data, $val);
-	}
 
-	my $stat = Statistics::Descriptive::Full->new();
-	$stat->add_data(@data);
-	$max = $stat->maxdex();
-	$min = $stat->mindex();
+	    my $stat = Statistics::Descriptive::Full->new();
+	    $stat->add_data(@data);
+	    return ($stat->maxdex(), $min = $stat->mindex());
+	};
+
+	if($len > 100000){
+	    my @cumgcskew = gcskew($gb, -output=>"/dev/null", -window=>10000, 
+				   -purine=>$purine, -keto=>$keto, -cumulative=>1);
+
+	    my $stat = Statistics::Descriptive::Full->new();
+	    $stat->add_data(@cumgcskew);
+	    my $maxi = $stat->maxdex();
+	    my $mini = $stat->mindex();
+
+	    ($max, undef) = peak_search(substr($gb->{SEQ}, $maxi * 10000, 20000));
+	    (undef, $min) = peak_search(substr($gb->{SEQ}, $mini * 10000, 20000));
+	    $max += $maxi * 10000;
+	    $min += $mini * 10000;
+	    $max = 0 if ($max < $len / 1000 || abs($max - $len) < $len / 1000);
+	}else{
+	    ($max, $min) = peak_search($gb->{SEQ});
+	}
     }
 
     &msg_send("   Predicted Origin:   " , $max, "\n")  if ($output eq 'stdout');
@@ -296,10 +317,18 @@ sub rep_ori_ter {
 
  Description:
    This method returns the leading strands from origin and terminus 
-   of replication calculated with rep_ori_ter()
+   of replication calculated with rep_ori_ter().
+
+   When called in array context, this method returns the sequences
+   of the two replication arms. In scalar context, the sequences of
+   the two arms are concatenated and returned as one sequence.
 
  Usage: 
-    (string seq1, string seq2) = &leading_strand(G instance);
+    #in array context
+    ($arm1, $arm2) = leading_strand($genome);
+
+    #in scalar context
+    string $leadingStrand = leading_strand($genome);
 
  Options:
     none
@@ -308,6 +337,7 @@ sub rep_ori_ter {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20071022-01 added array/scalar contexts
     20011030-01 initial posting
 
 =cut
@@ -329,7 +359,11 @@ sub leading_strand {
 	$seq2 = complement( substr($gb->{SEQ}, $ter) . substr($gb->{SEQ}, 0, $ori) );
     }
     
-    return ($seq1, $seq2);
+    if(wantarray()){
+	return ($seq1, $seq2);
+    }else{
+	return $seq1 . $seq2;
+    }
 }
 
 
@@ -365,7 +399,7 @@ sub leading_strand {
 
   References:
     1. Arakawa K, Tomita M (2007) "The GC skew index: a measure of genomic compositional 
-       asymmetry and the degree of replicational selection", Evolutionary Bioinformatics, in press.
+       asymmetry and the degree of replicational selection", Evolutionary Bioinformatics, 3:145-154.
 
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
@@ -572,7 +606,7 @@ sub gcskew {
 sub cum_gcskew {
     msg_error("WARNING: cum_gcskew is deprecated since v.1.6.13.\n" .
 	      "         This method will be removed in future releases.\n" .
-	      "         use gcskew($gb, -cumulative=>1) instead!\n\n");
+	      "         use gcskew(\$gb, -cumulative=>1) instead!\n\n");
 
     return gcskew(@_, -cumulative=>1);
 }
