@@ -35,6 +35,8 @@ use SubOpt;
 use G::Messenger;
 use G::Tools::GPAC;
 use G::Tools::GMap;
+use G::Tools::Statistics;
+use G::Seq::GCskew;
 use G::DB::SDB;
 
 use strict;
@@ -49,10 +51,12 @@ require Exporter;
 our @ISA = qw(Exporter);
 
 our @EXPORT = qw(
+		 dnawalk
 		 genome_map
 		 genome_map2
 		 genome_map3	 
 		 plasmid_map
+		 circular_map
 		 seq2png
 		 );
 
@@ -121,6 +125,11 @@ my %COG_fcolor = (
 		  '-'=>"aliceblue"
 		  );
 
+my $cx = 4000;
+my $cy = 4000;
+my $pi = 4 * atan2(1,1);
+
+
 __DATA__
 
 #::::::::::::::::::::::::::::::
@@ -435,6 +444,8 @@ sub genome_map {
 
 =head2 genome_map2
 
+  Name: genome_map2   -   draws the map of the genome (version 2)
+
   Description:
     This method creates a map of the genome, showing the local 
     nucleotide contents and positions of genes using SVG.
@@ -705,7 +716,7 @@ sub genome_map2 {
 			       });
 	    }else{
 		$svg->rect(
-			   id=>"liner$cds", x=>($sidemargin + ($cdsstart - $start)/$window), 
+			   id=>"liner2$cds", x=>($sidemargin + ($cdsstart - $start)/$window), 
 			   y=>($topmargin + $vblock * 5 + $hblock / 2 - 1 + $hblock * $col),
 			   height=>(abs($dif1) + 3),
 			   width=>(($cdsend - $cdsstart)/$window),
@@ -893,18 +904,21 @@ sub genome_map2 {
     -height         image height (default: 8192)
     -output         "g" for graph, "show" for display (default: "show") 
     -datafilename   coordinate file name (default: "data.coord")
+    -gmap           1 to create Google Map view, 0 to only generate PNG image
+                    (default:1)
 
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+   20071008-01 added -gmap option
    20070628-01 added -datafilename option
    20070623-01 initial posting
 
 =cut
 
 sub genome_map3 {
-    &opt_default(output=>"show", width=>8192, height=>8192, datafilename=>"data.coord");
+    &opt_default(output=>"show", width=>8192, height=>8192, datafilename=>"data.coord", gmap=>1);
     my @args = opt_get(@_);
     my $gb = opt_as_gb(shift @args);
     my $acnum = $gb->{LOCUS}->{id};
@@ -917,6 +931,7 @@ sub genome_map3 {
     my $vblock = 10;
     my $width = opt_val("width");
     my $height = opt_val("height");
+    my $gmap = opt_val("gmap");
 
     my $numHBlock = int(($width - $sidemargin - 50) / $hblock);
     my $numVBlock = int(($height - $topmargin - 50) / ($vblock * 5));
@@ -1099,12 +1114,12 @@ sub genome_map3 {
 	    }
 
 	    if($cdsdir eq 'complement'){
-		printf OUT "%s\t%s\t%s\t%s\t%s\n", $genename, $sidemargin + ($cdsstart / $window) % ($hblock * $numHBlock),
+		printf OUT "%s\t%s\t%s\t%s\t%s\n", $feat, $sidemargin + ($cdsstart / $window) % ($hblock * $numHBlock),
 		$topmargin + $dif1 + (int(($cdsstart / $window) / ($hblock * $numHBlock))+1) * $vblock * 5,
 		$sidemargin + ($cdsend / $window) % ($hblock * $numHBlock),
 		$topmargin + $dif2 + (int(($cdsend / $window) / ($hblock * $numHBlock))+1) * $vblock * 5;
 	    }else{
-		printf OUT "%s\t%s\t%s\t%s\t%s\n", $genename, $sidemargin + ($cdsstart / $window) % ($hblock * $numHBlock),
+		printf OUT "%s\t%s\t%s\t%s\t%s\n", $feat, $sidemargin + ($cdsstart / $window) % ($hblock * $numHBlock),
 		$topmargin + $dif2 + (int(($cdsstart / $window) / ($hblock * $numHBlock))+1) * $vblock * 5,
 		$sidemargin + ($cdsend / $window) % ($hblock * $numHBlock),
 		$topmargin + $dif1 + (int(($cdsend / $window) / ($hblock * $numHBlock))+1) * $vblock * 5;
@@ -1159,7 +1174,7 @@ sub genome_map3 {
     print IMAGE $im->png;
     close(IMAGE);
 
-    generateGMap("graph/$acnum.png");
+    generateGMap("graph/$acnum.png") if($gmap);
 }
 
 
@@ -1169,6 +1184,8 @@ sub genome_map3 {
 
 =head2 seq2png
 
+ Name: seq2png   -   converts a sequence to PNG image
+
  Description:
   Converts a sequence to a png image, by representing nucleotide 
   sequences with representative pixels. 
@@ -1256,8 +1273,11 @@ sub seq2png {
 
 =head2 plasmid_map
 
+  Name: plasmid_map   -   draws circular map of the genome
+
   Description:
-   This method creates a plasmid map image with SVG.
+   This method creates a circular map of the genome using SVG,
+   suitable for plasmids and circular bacterial chromosomes.
 
   Usage:
     NULL = plasmid_map($gb);
@@ -1271,6 +1291,7 @@ sub seq2png {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+   20071023-01 fixed SVG-related bug
    20020905-01 initial posting
 
 =cut
@@ -1304,7 +1325,7 @@ sub plasmid_map{
     }
     
     foreach my $cds ($gb->cds()){
-	my $id = $gb->{$cds}->{gene};
+	my $id = $cds;
 	my $gene = $gb->get_geneseq($cds);
 	my $length = length $gene;
 	my $gc = int(log($maxlen) / log($length) * 30);
@@ -1411,19 +1432,19 @@ sub plasmid_map{
 				  );
     
     $svg->text(
-	       id=>"locus", 
+	       id=>"locus1", 
 	       x=>30, y=>30, fill=>"navy", 
 	       'font-size'=>10,
 	       )->cdata("Accession Number: ", $gb->{LOCUS}->{id});
     
     $svg->text(
-	       id=>"locus", 
+	       id=>"locus2", 
 	       x=>250, y=>30, fill=>"navy", 
 	       'font-size'=>10,
 	       )->cdata("Organism: ", $gb->{FEATURE0}->{organism});
     
     $svg->text(
-	       id=>"locus", 
+	       id=>"locus3", 
 	       x=>500, y=>30, fill=>"navy", 
 	       'font-size'=>10,
 	       )->cdata("$seqlen bp");
@@ -1441,6 +1462,655 @@ sub plasmid_map{
 
 
 
+=head2 circular_map
 
+  Name: circular_map   -   draws circular map of the genome
+
+  Description:
+   This method creates a circular map of the genome using SVG,
+   suitable for plasmids and circular bacterial chromosomes.
+
+   From the outer ring inwards, genes on direct strand (pink), 
+   genes on complementary strand (yellow), tRNAs (green arrows), 
+   rRNAs (pink or orange stripes depending on the strand), 
+   GC content (brown lines), GC skew (yellow lines). Replication 
+   origin and terminus predicted from the GC skew shift points 
+   are also labeled. 
+
+   Generated SVG is very large (8000x8000 pixels), and it is
+   suited to be converted to Google Map View with generate_gmap().
+
+   Output file is created in the "graph" directory, with 
+   the accession number as file name and ".svg" as extension.
+
+  Usage:
+    NULL = circular_map($gb);
+
+ Options:
+    None.
+
+  Author: 
+    Nobuhiro Kido (t07505nk@sfc.keio.ac.jp)
+
+  History:
+   20071023-01 initial posting
+
+=cut
+
+
+sub circular_map{
+    my $gb = opt_as_gb(shift);
+
+    mkdir( "data", 0777 );
+    mkdir( "graph",  0777 );
+
+    my $wfile = $gb->{LOCUS}->{id};
+    my ( $ori, $ter ) = find_ori_ter( $gb, -output => "/dev/null" );
+    my $bp_length = length( $gb->{SEQ} );
+    my ( $count_a, $count_b );
+
+    my $svg = SVG->new(
+        width         => $cx * 2,
+        height        => $cy * 2,
+        'xmlns:xlink' => 'http://www.w3.org/1999/xlink'
+    );
+
+    my $tag = $svg->rect(
+        width  => $cx * 2,
+        height => $cy * 2,
+        x      => 0,
+        y      => 0,
+        fill   => 'black'
+    );
+
+    my $y = $svg->group(
+        id => 'group_y',
+        style =>
+          { 'stroke' => '#ff9966', 'fill' => 'none', 'stroke-width' => 200 }
+    );
+
+    #$y->circle(cx=>$cx, cy=>$cy,r=>6500, id=>'main_c');
+
+    my $scla = $svg->group(
+        id => 'scale_a',
+        style =>
+          { 'stroke' => '#333333', 'fill' => 'none', 'stroke-width' => 1 }
+    );
+    my $sclb = $svg->group(
+        id => 'scale_b',
+        style =>
+          { 'stroke' => '#ffffff', 'fill' => 'none', 'stroke-width' => 2 }
+    );
+    my $sclc = $svg->group(
+        id => 'scale_c',
+        style =>
+          { 'stroke' => '#ffffff', 'fill' => 'none', 'stroke-width' => 2 }
+    );
+
+    my $cdsa = $svg->group(
+        id => 'cds_a',
+        style =>
+          { 'stroke' => '#cc0066', 'fill' => 'none', 'stroke-width' => 300 }
+    );
+    my $cdsb = $svg->group(
+        id => 'cds_b',
+        style =>
+          { 'stroke' => '#cc0033', 'fill' => 'none', 'stroke-width' => 300 }
+    );
+    my $cdsx = $svg->group(
+        id => 'cds_x',
+        style =>
+          { 'stroke' => '#ccff33', 'fill' => 'none', 'stroke-width' => 300 }
+    );
+    my $cdsy = $svg->group(
+        id => 'cds_y',
+        style =>
+          { 'stroke' => '#99cc00', 'fill' => 'none', 'stroke-width' => 300 }
+    );
+    my $cdst = $svg->group( id => 'cds_texts', style => { fill => '#ccccff' } );
+
+    my $trnaa = $svg->group(
+        id => 'trna_a',
+        style =>
+          { 'stroke' => '#3399ff', 'fill' => 'none', 'stroke-width' => 200 }
+    );
+    my $trnax = $svg->group(
+        id => 'trna_x',
+        style =>
+          { 'stroke' => '#00ffcc', 'fill' => 'none', 'stroke-width' => 200 }
+    );
+
+    my $yaji = $svg->group(
+        id => 'yajirushi',
+        style =>
+          { 'stroke' => '#ccffcc', 'fill' => 'none', 'stroke-width' => 5 }
+    );
+
+    my $rrnaa = $svg->group(
+        id => 'rrna_a',
+        style =>
+          { 'stroke' => '#ff6699', 'fill' => 'none', 'stroke-width' => 200 }
+    );
+    my $rrnax = $svg->group(
+        id => 'rrna_x',
+        style =>
+          { 'stroke' => '#ff9933', 'fill' => 'none', 'stroke-width' => 200 }
+    );
+
+    my $gcska = $svg->group(
+        id => 'gccontent',
+        style =>
+          { 'stroke' => '#ff9933', 'fill' => 'none', 'stroke-width' => 3 }
+    );
+    my $gcskb = $svg->group(
+        id => 'gcskew',
+        style =>
+          { 'stroke' => '#ccff66', 'fill' => 'none', 'stroke-width' => 3 }
+    );
+
+    my $oriter = $svg->group(
+        id => 'ori_ter',
+        style =>
+          { 'stroke' => '#cccc33', 'fill' => 'none', 'stroke-width' => 20 }
+    );
+
+
+    my $title = $gb->{FEATURE0}->{organism};
+
+    my $text1 = $svg->text(
+        id            => 'title',
+        'text-anchor' => 'middle',
+        x             => $cx,
+        y             => $cy,
+        fill          => '#cccccc',
+        'font-size'   => 200,
+        'font-family' => 'Gill Sans'
+    )->cdata($title);
+
+    open( PWRITER, ">./data/" . $wfile . ".coord" );
+
+    my ( $xe, $ye ) = &get_pos( $ori, 4000, $bp_length );
+    $oriter->path( 'd' => "M$cx,$cy L$xe,$ye" );
+
+    if ( $xe < $cx ) {
+        $cdst->text(
+            'text-anchor' => 'end',
+            x             => $xe,
+            y             => $ye,
+            'font-size'   => 100
+        )->cdata("ORIGIN");
+    }
+    else {
+        $cdst->text( x => $xe, y => $ye, 'font-size' => 100 )->cdata("ORIGIN");
+    }
+
+    my ( $xe, $ye ) = &get_pos( $ter, 4000, $bp_length );
+    $oriter->path( 'd' => "M$cx,$cy L$xe,$ye" );
+
+    if ( $xe < $cx ) {
+        $cdst->text(
+            'text-anchor' => 'end',
+            x             => $xe,
+            y             => $ye,
+            'font-size'   => 100
+        )->cdata("TERMINUS");
+    } else {
+        $cdst->text( x => $xe, y => $ye, 'font-size' => 100 )->cdata("TERMINUS");
+    }
+
+    my $num = $bp_length;
+    while ( $num =~ s/(.*\d)(\d\d\d)/$1,$2/ ) { }
+
+    my $text2 = $svg->text(
+        id            => 'bplength',
+        'text-anchor' => 'middle',
+        x             => $cx,
+        y             => $cy + 400,
+        fill          => '#cccccc',
+        'font-size'   => 150,
+        'font-family' => 'Gill Sans'
+    )->cdata( $num . " bp" );
+
+    my $i = 0;
+    $sclb->circle( cx => $cx, cy => $cy, r => 3750 );
+    $sclc->circle( cx => $cx, cy => $cy, r => 3680 );
+
+    while ( $i < $bp_length ) {
+        my ( $xe, $ye ) = &get_pos( $i, 3700, $bp_length );
+        $scla->path( 'd' => "M$cx,$cy L$xe,$ye" );
+        $cdst->text( x => $xe, y => $ye )->cdata( $i . "bp" );
+        my ( $xe, $ye ) = &get_pos( $i, 2850, $bp_length );
+        $cdst->text( x => $xe, y => $ye )->cdata( $i . "bp" );
+        my ( $xe, $ye ) = &get_pos( $i, 1800, $bp_length );
+        $cdst->text( x => $xe, y => $ye )->cdata( $i . "bp" );
+
+        if ( $i % 250000 == 0 ) {
+            my ( $xs, $ys ) = &get_pos( $i, 3630, $bp_length );
+            my ( $xe, $ye ) = &get_pos( $i, 3680, $bp_length );
+            $sclc->path( 'd' => "M$xs,$ys L$xe,$ye" );
+
+            $cdst->text(
+                'text-anchor' => 'middle',
+                x             => $xe,
+                y             => $ye,
+                'font-size'   => 80
+            )->cdata( $i . "bp" );
+        }
+        $i = $i + 25000;
+    }
+    for ( $i = 0 ; $i < 100 ; $i++ ) {
+        my ( $xs, $ys ) = &get_posp( $i, 3700, $bp_length );
+        my ( $xe, $ye ) = &get_posp( $i, 3750, $bp_length );
+        $sclc->path( 'd' => "M$xs,$ys L$xe,$ye" );
+
+        if ( $i % 10 == 0 ) {
+            if ( $xe < $cx ) {
+                $cdst->text(
+                    'text-anchor' => 'end',
+                    x             => $xe,
+                    y             => $ye,
+                    'font-size'   => 80
+                )->cdata( $i, "\%" );
+            } else {
+                $cdst->text( x => $xe, y => $ye, 'font-size' => 80 )->cdata( $i, "\%" );
+            }
+        }
+    }
+
+    for my $feature ( $gb->feature() ) {
+        if ( $gb->{$feature}->{type} eq "gene" and $gb->{$feature}->{direction} eq "direct" ) {
+            $count_a++;
+            my $r = 3400;
+            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+
+            if ( $count_a % 2 == 1 ) {
+                $cdsa->path(
+                    id  => "a" . $count_a,
+                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
+                );
+            } else {
+                $cdsb->path(
+                    id  => "a" . $count_a,
+                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
+                );
+            }
+
+            my ( $xm, $ym ) = &get_mapp( ( $xs + $xe ) / 2, ( $ys + $ye ) / 2, $bp_length );
+            print PWRITER "$feature\t$gb->{$feature}->{gene}\t$xm\t$ym\n";
+
+        }
+
+        if ( $gb->{$feature}->{type} eq "gene" and $gb->{$feature}->{direction} eq "complement" ){
+            $count_b++;
+            my $r = 3100;
+            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+
+            if ( $count_b % 2 == 1 ) {
+                $cdsx->path(
+                    id  => "b" . $count_b,
+                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
+                );
+            } else {
+                $cdsy->path(
+                    id  => "b" . $count_b,
+                    'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye"
+                );
+            }
+
+            my ( $xm, $ym ) = &get_mapp( ( $xs + $xe ) / 2, ( $ys + $ye ) / 2, $bp_length );
+            print PWRITER "$feature\t$gb->{$feature}->{gene}\t$xm\t$ym\n";
+
+        }
+
+        if ( $gb->{$feature}->{type} eq "tRNA" and $gb->{$feature}->{direction} eq "direct" ){
+            my $r = 2600;
+            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+            $trnaa->path( 'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye" );
+            my ( $xxs, $yys, $xxe, $yye, $xl, $yl, $xr, $yr ) = &get_arrow( $gb->{$feature}->{start}, $r + 50, $bp_length );
+            $yaji->path( 'd' => "M$xxs $yys L$xxe $yye" );
+            $yaji->path( 'd' => "M$xxe $yye L$xl $yl" );
+            $yaji->path( 'd' => "M$xxe $yye L$xr $yr" );
+        }
+
+        if ( $gb->{$feature}->{type} eq "tRNA" and $gb->{$feature}->{direction} eq "complement" ){
+            my $r = 2600;
+            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+            $trnax->path( 'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye" );
+            my ( $xxs, $yys, $xxe, $yye, $xl, $yl, $xr, $yr ) = &get_arrowc( $gb->{$feature}->{start}, $r - 50, $bp_length );
+            $yaji->path( 'd' => "M$xxs $yys L$xxe $yye" );
+            $yaji->path( 'd' => "M$xxs $yys L$xl $yl" );
+            $yaji->path( 'd' => "M$xxs $yys L$xr $yr" );
+        }
+
+        if ( $gb->{$feature}->{type} eq "rRNA" and $gb->{$feature}->{direction} eq "direct" ){
+            my $r = 2400;
+            my ( $xs, $ys ) = &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            my ( $xe, $ye ) = &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+            $rrnaa->path( 'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye" );
+        }
+
+        if ( $gb->{$feature}->{type} eq "rRNA" and $gb->{$feature}->{direction} eq "complement" ){
+            my $r = 2400;
+            my ( $xs, $ys ) =
+              &get_pos( $gb->{$feature}->{start}, $r, $bp_length );
+            my ( $xe, $ye ) =
+              &get_pos( $gb->{$feature}->{end}, $r, $bp_length );
+            $rrnax->path( 'd' => "M$xs $ys A$r $r,0,0 1,$xe $ye" );
+        }
+    }
+
+    my $bline  = 2000;
+    my $blineg = 1500;
+    my ( $px,  $py )  = &get_pos( 0, $bline,  $bp_length );
+    my ( $pxg, $pyg ) = &get_pos( 0, $blineg, $bp_length );
+
+    my $range = 1000;
+    my $wsize = 2000;           #window_bp_size
+    my $step  = 1000;
+
+    my $tag   = $svg->circle(
+        cx     => $cx,
+        cy     => $cy,
+        r      => $bline,
+        stroke => '#33ff99',
+        fill   => 'none'
+    );
+
+    my $tag = $svg->circle(
+        cx     => $cx,
+        cy     => $cy,
+        r      => $blineg,
+        stroke => '#66ffff',
+        fill   => 'none'
+    );
+
+    for ( my $i = 0 ; $i + $wsize <= $bp_length ; $i += $step ) {
+        my $temp = substr( $gb->{SEQ}, $i, $wsize );
+        my $c_g = $temp =~ tr/g/g/;
+        my $c_c = $temp =~ tr/c/c/;
+        my $c_a = $temp =~ tr/a/a/;
+        my $c_t = $temp =~ tr/t/t/;
+
+        my $gc_content = 0;
+        $gc_content = ( $c_g + $c_c ) / ( $c_g + $c_c + $c_a + $c_t )
+          if ( $c_g + $c_c + $c_a + $c_t );
+
+        my ( $x, $y ) = &get_pos( $i, ( $bline + ( $gc_content - 0.5 ) * $range * 2 ), $bp_length );
+
+        my $gc_skew = 0;
+        $gc_skew = ( $c_c - $c_g ) / ( $c_g + $c_c ) if ( $c_g + $c_c );
+
+        my ( $xg, $yg ) = &get_pos( $i, ( $blineg + ($gc_skew) * $range * 2 ), $bp_length );
+
+        $gcska->path( 'd' => "M$px,$py L$x,$y" );
+
+        $px = $x;
+        $py = $y;
+        $gcskb->path( 'd' => "M$pxg,$pyg L$xg,$yg" );
+
+        $pxg = $xg;
+        $pyg = $yg;
+    }
+
+    $cdst->text(
+        'text-anchor' => 'middle',
+        x             => $cx,
+        y             => 1750,
+        'font-size'   => 120
+    )->cdata("GC-contents");
+
+    $cdst->text(
+        'text-anchor' => 'middle',
+        x             => $cx,
+        y             => 3000,
+        'font-size'   => 120
+    )->cdata("GC-skew");
+
+    close PWRITER;
+    open( WRITER, ">./graph/" . $wfile . ".svg" );
+    print WRITER $svg->xmlify;
+    close WRITER;
+}
+
+sub get_pos {
+    my ( $n, $r, $bp_length ) = @_;
+    my $x = $cx + ( sin( 2 * $pi * $n / $bp_length ) * $r );
+    my $y = $cy - ( cos( 2 * $pi * $n / $bp_length ) * $r );
+
+    return ( $x, $y );
+}
+
+sub get_posp {
+    my ( $n, $r, $bp_length ) = @_;
+    my $x = $cx + ( sin( 2 * $pi * $n / 100 ) * $r );
+    my $y = $cy - ( cos( 2 * $pi * $n / 100 ) * $r );
+
+    return ( $x, $y );
+}
+
+sub get_mapp {
+    my ( $n, $r, $bp_length ) = @_;
+    my $x = $n * 8192 / 8000;
+    my $y = $r * 8192 / 8000;
+
+    return ( $x, $y );
+}
+
+sub get_arrow {
+    my ( $n, $r, $bp_length ) = @_;
+
+    my $rr = 40;                                                #length
+    my $rl = 20;                                                #hand_length
+    my $x  = $cx + ( sin( 2 * $pi * $n / $bp_length ) * $r );
+    my $y  = $cy - ( cos( 2 * $pi * $n / $bp_length ) * $r );
+    my $th  = 2 * $pi * ( 1 + $n ) / $bp_length;
+    my $xxs = $x - cos($th) * $rr;
+    my $yys = $y - sin($th) * $rr;
+    my $xxe = $x + cos($th) * $rr;
+    my $yye = $y + sin($th) * $rr;
+
+    $th -= $pi * 3 / 4;
+    my $xl = $xxe + cos($th) * $rl;
+    my $yl = $yye + sin($th) * $rl;
+    my $th -= $pi * 2 / 4;
+    my $xr = $xxe + cos($th) * $rl;
+    my $yr = $yye + sin($th) * $rl;
+
+    return ( $xxs, $yys, $xxe, $yye, $xl, $yl, $xr, $yr );
+}
+
+sub get_arrowc {
+    my ( $n, $r, $bp_length ) = @_;
+
+    my $rr = 40;                                                #length
+    my $rl = 20;                                                #hand_length
+    my $x  = $cx + ( sin( 2 * $pi * $n / $bp_length ) * $r );
+    my $y  = $cy - ( cos( 2 * $pi * $n / $bp_length ) * $r );
+    my $th  = 2 * $pi * ( 1 + $n ) / $bp_length;
+    my $xxs = $x - cos($th) * $rr;
+    my $yys = $y - sin($th) * $rr;
+    my $xxe = $x + cos($th) * $rr;
+    my $yye = $y + sin($th) * $rr;
+    
+    $th -= $pi * 1 / 4;
+    my $xl = $xxs + cos($th) * $rl;
+    my $yl = $yys + sin($th) * $rl;
+    my $th += $pi * 2 / 4;
+    my $xr = $xxs + cos($th) * $rl;
+    my $yr = $yys + sin($th) * $rl;
+
+    return ( $xxs, $yys, $xxe, $yye, $xl, $yl, $xr, $yr );
+}
+
+
+
+
+
+
+
+
+
+
+=head2 dnawalk
+
+  Name: dnawalk   -   draws DNA Walk map of the genome
+
+  Description:
+   This method draws the DNA Walk map of the given genome.
+   DNA Walk is drawn by moving a single pixel per nucleotide, 
+   in the direction specified for each base. Here A is moved
+   upward, T downward, G to the right, and C to the left.
+   Position zero (first letter of the genome) is indicated
+   by the crossing of thin axes.
+   
+   Generated PNG is very large (8000x8000 pixels), and it is
+   suited to be converted to Google Map View with generate_gmap().
+
+   Output file is created in the "graph" directory, with 
+   the accession number as file name and ".png" as extension.
+
+  Usage:
+    NULL = dnawalk($gb);
+
+ Options:
+    None.
+
+  Author: 
+    Keita Ikegami (t06056ki@sfc.keio.ac.jp)
+
+  History:
+   20071023-01 initial posting
+
+=cut
+
+
+sub dnawalk{
+    my $gb = opt_as_gb(shift);
+    my $name = $gb->{LOCUS}->{id};
+    my $segments = 10;
+
+    mkdir("data", 0777);
+    mkdir("graph", 0777);
+    
+    my ($XX, $YY) = (8192, 8192);
+    my $im = new GD::Image($XX,$YY);    
+    my $margin = 50;
+    
+    my ($cds_x, $cds_y);
+    my ($out, @out);
+    
+    my $black   = $im->colorAllocate(0  ,0  ,0  ); ### black
+    my $white   = $im->colorAllocate(255,255,255); ### white
+    my $momo    = $im->colorAllocate(255,127,127); ### momo
+    my $h_p     = $im->colorAllocate(255,127,191); ### hard_pink
+    my $s_p     = $im->colorAllocate(255,127,255); ### soft_pink
+    my $ppl     = $im->colorAllocate(191,127,255); ### purple
+    my $h_b     = $im->colorAllocate(127,127,255); ### hard_blue
+    my $s_b     = $im->colorAllocate(127,191,255); ### soft_blue
+    my $ss_b    = $im->colorAllocate(127,255,255); ### soft+soft_blue
+    my $s_g     = $im->colorAllocate(127,255,191); ### soft_green
+    my $h_g     = $im->colorAllocate(127,255,127); ### hard_green
+    my $g_y     = $im->colorAllocate(191,255,127); ### green_yallow
+    my $sb_y    = $im->colorAllocate(255,255,127); ### sibui_yellow
+    my $p_y     = $im->colorAllocate(255,191,127); ### pink_yallew
+    my $black   = $im->colorAllocate(0,  0,  0, ); ### black
+    
+    my @colors = ($momo, $h_p, $s_p, $ppl, $h_b, $s_b, $ss_b, $s_g, $h_g, $g_y);
+    
+    my ( $x, $y, $color ) = ( 0, 0, $black );
+    my ( @pos_x, @pos_y) ;
+    my $nuc;
+    
+    for (my $i=0; $i<=$segments; $i++){
+	my $tmpseq = substr( $gb->{SEQ},  $i * int(length($gb->{SEQ})/$segments),  int(length($gb->{SEQ})/$segments) );
+	$tmpseq =~ tr[atgc][0286];
+	
+	foreach my $word (split(//, $tmpseq)){
+	    $nuc++;
+	    
+	    if($word < 5){
+		$y += ($word - 1);
+	    }else{
+		$x += ($word - 7);
+	    }
+	    
+	    push (@pos_x, $x);
+	    push (@pos_y, $y); 
+	}
+    }
+    
+    my $walk_width  = max(@pos_x) - min(@pos_x);
+    my $walk_height = max(@pos_y) - min(@pos_y);
+
+    my $compress;
+    my ($cx, $cy);
+    
+    if( $walk_width > $walk_height ){
+	$compress = $walk_width / (8192 - $margin);
+	
+	if( $compress < 1  &&  $compress > 0 ){
+	    $compress = 1;
+	    
+	    $cx = int (-min(@pos_x)/ $compress + ( 8192 - $walk_width/$compress) / 2);
+	    $cy = int (-min(@pos_y)/ $compress + ( 8192 - $walk_height/$compress)/ 2) ;
+	}else{
+	    $cx = int (- min(@pos_x) / $compress) + $margin/2;
+	    $cy = int (- min(@pos_y) / $compress + ( 8192 - $walk_height/$compress) / 2);
+	}
+
+    }else{
+	
+	$compress = $walk_height / (8192 - $margin);
+	
+	if( $compress < 1  &&  $compress > 0 ){
+	    $compress = 1;
+	    
+	    $cx = int (-min(@pos_x)/ $compress + ( 8192 - $walk_width/$compress) / 2);
+	    $cy = int (-min(@pos_y)/ $compress + ( 8192 - $walk_height/$compress)/ 2) ;
+	}else{
+	    $cx = int (-min(@pos_x)/ $compress + ( 8192 - $walk_width/$compress) / 2);
+	    $cy = int (-min(@pos_y) / $compress) +$margin/2;
+	}
+    }
+    
+    $im->line(0,   $cy, 8192, $cy,  $white); # yoko
+    $im->line($cx, 0,   $cx,  8192, $white); # tate
+    
+    my $j = 0;
+    my $k = 0;
+    
+    for (my $i=0; $i<=length($gb->{SEQ}); $i++){
+	$j++;
+	if ($j > length($gb->{SEQ}) / $segments){
+	    $k++;
+	    $j = 0;
+	}
+	$color = $colors[$k];
+
+	my $xs = int( $cx + $pos_x[$i] / $compress );
+	my $ys = int( $cy + $pos_y[$i] / $compress );
+	
+	$im->setPixel( $xs, $ys, $color );
+    }
+    
+    open DATA, ">./data/$name.coord";
+    
+    for my $cds ($gb->feature()){
+	$cds_x = int ($cx + @pos_x[$gb->{$cds}->{start}] / $compress);
+	$cds_y = int ($cy + @pos_y[$gb->{$cds}->{start}] / $compress);
+	my $gene = $gb->{$cds}->{gene} or $gb->{$cds}->{locus_tag};
+	print DATA "$cds\t$gene\t$cds_x\t$cds_y\n";	
+    }
+    close DATA;   
+    
+    open NEW, ">./graph/" . $gb->{LOCUS}->{id} . ".png"; #automatically name the file
+    binmode NEW;
+    print NEW $im->png;
+    close NEW;
+}
 
 1;
