@@ -83,9 +83,42 @@ perl(1).
 ######################################
 
 
-# set_operon ver. 20020207-01
-# modified 20061003 to use RegulonDB
-# Author: Kazuharu Arakawa
+=head2 set_operon
+
+  Name: set_operon   -   set operon information from RegulonDB
+
+  Description:
+    This program retrieves the operon information from RegulonDB, and adds this
+    to the given genome data. !!!This method currently only works for E.coli!!!
+
+    Two attributes are added to each CDS hash.
+        $genome->{$cds}->{operon}
+    contains the name of the operon to which the gene belongs, and 
+        $genome->{$cds}->{operonN}
+    contains the rank order of the gene within the operon.
+
+  Usage:
+    set_operon($gb);
+
+ Options:
+   None.
+
+  References:
+   1. Salgado H et al. (2006) "RegulonDB (version 5.0): Escherichia coli K-12 transcriptional
+      regulatory network, operon organization, and growth conditions", Nucleic Acids Res. 
+      1;34(Database issue):D394-7
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+   20070829-01 patched to match latest version of RegulonDB formamt
+               (patch by Hiroyuki Nakamura <t04632hn@sfc.keio.ac.jp>
+   20061003-01 updated to use data from RegulonDB
+   20020207-01 initial posting
+
+=cut
+
 
 sub set_operon {
     my @args = opt_get(@_);
@@ -100,25 +133,29 @@ sub set_operon {
 
 	my $flag = 0;
 	open(FILE, $dir) || die($!);
-	while(<FILE>){
+	while (<FILE>) {
 	    chomp;
-	    
-	    if(/^\(1\)/){
-		$flag ++;
-		next;
+
+	    if (/^Columns\:/) {
+	      $flag++;
+	      next;
 	    }
-	    
-	    if($flag){
+	    elsif(/^\t\(\d\)\s/) {
+	      $flag++;
+	      next;
+	    }
+
+	    if($flag == 5){
 
 		my %geneOrder;
 
 		my ($operon, $num, $direction, $genes) = split(/\t/, $_, 4);
 		next unless($num >= 2);
-		
+
 		foreach my $genepair (split(/,/, $genes)){
 		    my ($gene, $locustag) = split(/\|/, $genepair, 2);
-		    
 		    my $cds = $gb->gene2id($locustag);
+
 		    $cds = $gb->gene2id($gene) unless(length $cds);
 
 		    if($cds){
