@@ -111,7 +111,7 @@ our @EXPORT = qw(
 
 
 sub generateGMap{
-    opt_default(level=>6);
+    opt_default(level=>6, format=>"gif");
     my @argv = opt_get(@_);
     my $file = shift @argv;
     my $level = opt_val("level");
@@ -148,10 +148,9 @@ sub generateGMap{
 	    $size = $height;
 	    $xdif = int(($height - $width) / 2);
 	}
-	
+	print "HAGE";
 	system(sprintf("convert +dither -posterize 16 -extent %dx%d -roll +%s+%s %s %s/%s.png", $size, $size, $xdif, $ydif, $file, $name, $name));
     }
-    
     chdir($name);
     
     for(my $i = 0; $i < $level; $i ++){
