@@ -34,7 +34,7 @@ package G::Tools::GlimmerM;
 
 use strict;
 use G::Messenger;
-use G::Skyline;
+use G::IO;
 
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK @INC $AUTOLOAD);
 
@@ -68,7 +68,7 @@ sub annotate_with_glimmerM {
     my $file = shift;
     my $out = shift;
 
-    my $this = new G::Skyline("blessed");    
+    my $this = new G::IO("blessed");    
     open (FASTA, $file);
     while(<FASTA>){
 	if (/^\>/){
